#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_base import Node_Base
from homie.node.property.property_speed import Property_Speed
import logging

logger = logging.getLogger(__name__)


class Device_Speed(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None,speeds ='OFF,LOW,MEDIUM,HIGH'):

        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        node = Node_Base(self,id='speed',name='Speed',type_='speed')
        self.add_node(node)

        self.speed_property = Property_Speed(node,data_format=speeds,set_value=self.set_speed)
        node.add_property(self.speed_property)

        self.start()

    def update_speed(self,speed):
        self.speed_property.value = speed
        logger.debug ('Speed Update {}'.format(speed))

    def set_speed(self,speed):
        logger.debug ('Speed Set {}'.format(speed))




#!/usr/bin/env python

import logging

from homie.device_status import Device_Status
from homie.node.property.property_temperature import Property_Temperature

logger = logging.getLogger(__name__)

class Device_Temperature(Device_Status):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None, temp_units='F'):
        self.temp_units = temp_units

        super().__init__ (device_id, name, homie_settings, mqtt_settings)

    def register_status_properties(self, node):
        self.temperature = Property_Temperature(node, unit=self.temp_units)
        node.add_property(self.temperature)

    def update_temperature(self,temperature):
        logger.info ('Updated Temperature {}'.format(temperature))
        self.temperature.value = temperature




#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_boolean import Node_Boolean
import logging

logger = logging.getLogger(__name__)

class Device_Boolean(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None):
        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_Boolean(self,id='boolean',set_boolean=self.set_boolean))

        self.start()

    def update_boolean(self,boolean): #sends updates to clients
        self.get_node('boolean').update_boolean(boolean)
        logger.debug ('Boolean Update {}'.format(boolean))

    def set_boolean(self,boolean):#received commands from clients
        logger.debug ('Boolean Set {}'.format(boolean))

        




#!/usr/bin/env python

import paho.mqtt.client as mqtt_client

import traceback

from uuid import getnode as get_mac
from homie.mqtt.mqtt_base import MQTT_Base

import logging
logger = logging.getLogger(__name__)
logger.setLevel('INFO')

mqtt_logger = logging.getLogger('MQTT')
mqtt_logger.setLevel('INFO')


COONNECTION_RESULT_CODES = {
    0: 'Connection successful',
    1: 'Connection refused - incorrect protocol version',
    2: 'Connection refused - invalid client identifier',
    3: 'Connection refused - server unavailable',
    4: 'Connection refused - bad username or password',
    5: 'Connection refused - not authorised',
}

class PAHO_MQTT_Client (MQTT_Base): 

    def __init__(self, mqtt_settings):
        MQTT_Base.__init__(self,mqtt_settings)

        self.mqtt_client= None

    def connect(self):
        MQTT_Base.connect(self)

        self.mqtt_client = mqtt_client.Client(client_id=self.mqtt_settings['MQTT_CLIENT_ID'])
        self.mqtt_client.on_connect = self._on_connect
        self.mqtt_client.on_message = self._on_message
        #self.mqtt_client.on_publish = self._on_publish
        self.mqtt_client.on_disconnect = self._on_disconnect
        self.mqtt_client.enable_logger(mqtt_logger)
        self.mqtt_client.enable_logger()

        if self.mqtt_settings ['MQTT_USERNAME']:
            self.mqtt_client.username_pw_set(
                    self.mqtt_settings ['MQTT_USERNAME'],
                    password=self.mqtt_settings ['MQTT_PASSWORD']
            )

        try:
            self.mqtt_client.connect(
                self.mqtt_settings ['MQTT_BROKER'],
                port=self.mqtt_settings ['MQTT_PORT'],
                keepalive=self.mqtt_settings ['MQTT_KEEPALIVE'],
            )

            self.mqtt_client.loop_start()

        except Exception as e:
            logger.warning ('MQTT Unable to connect to Broker {}'.format(e))

    def publish(self, topic, payload, retain, qos):
        MQTT_Base.publish(self,topic,payload,retain,qos)
        self.mqtt_client.publish(topic, payload, retain=retain, qos=qos)
    
    def subscribe(self, topic, qos): #subclass to provide
        MQTT_Base.subscribe(self,topic,qos)
        self.mqtt_client.subscribe(topic,qos)

    def unsubscribe(self, topic): #subclass to provide
        MQTT_Base.unsubscribe(self,topic)
        self.mqtt_client.unsubscribe(topic)

    def set_will(self,will,topic,retain,qos):
        MQTT_Base.set_will(self,will,topic,retain,qos)
        self.mqtt_client.will_set(will,topic,retain,qos)

    def _on_connect(self,client, userdata, flags, rc):
        logger.debug("MQTT On Connect: {}".format(rc))       
        self.mqtt_connected = (rc == 0)

    def _on_message(self, client, userdata, msg):
        topic = msg.topic
        payload = msg.payload.decode("utf-8")
        MQTT_Base._on_message(self,topic,payload,msg.retain,msg.qos)

    def _on_disconnect(self,client,userdata,rc):
        self.mqtt_connected = False # note, change this uses the property setter, do not really need to catch this in the base class

        if rc > 0: #unexpected disconnect
            rc_text = 'Unknown result code {}'.format(rc)
            if rc in COONNECTION_RESULT_CODES:
                rc_text = COONNECTION_RESULT_CODES [rc]

            logger.warn ('MQTT Unexpected disconnection  {} {} {}'.format(client,userdata,rc_text))
        
        MQTT_Base._on_disconnect(self,rc)
        



#!/usr/bin/env python

import logging
import netifaces
import socket

logger = logging.getLogger(__name__)

''' or use psutils

import psutil

mac_addresses = []

nics = psutil.net_if_addrs()
nics.pop('lo') # remove loopback since it doesnt have a real mac address

for i in nics:
    for j in nics[i]:
        if j.family == 17:  # AF_LINK
            mac_addresses.append(j.address)
'''


class Network_Information(object):
    """Util for getting a interface' ip to a specific host and the corresponding mac address."""

    def __init__(self):
        self.ip_to_interface = self.__build_ip_to_interface_dict()

    def __build_ip_to_interface_dict(self):
        """Build a map of IPv4-Address to Interface-Name (like 'eth0')"""
        map = {}
        for interface in netifaces.interfaces():
            try:
                ifInfo = netifaces.ifaddresses(interface)[netifaces.AF_INET]
                for addrInfo in ifInfo:
                    addr = addrInfo.get('addr')
                    if addr:
                        map[addr] = interface
            except Exception:
                pass
        return map

    def get_local_ip(self, targetHost, targetPort):
        """Gets the local ip to reach the given ip.
        That can be influenced by the system's routing table.
        A socket is opened and closed immediately to achieve that."""
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect((targetHost, targetPort))
        except Exception as e:
            logger.exception("Cannot create socket to target " + targetHost + ":" + targetPort)
        else:
            ip = s.getsockname()[0]
            s.close()
        return ip

    def get_local_mac_for_ip(self, ip):
        """Get the mac address for that given ip."""
        logger.debug("Interfaces found: %s", self.ip_to_interface)
        logger.debug("Looking for IP: %s", ip)

        mac_addr = None
        if_name = self.ip_to_interface.get(ip)

        try:
            link = netifaces.ifaddresses(if_name)[netifaces.AF_LINK]
        except (KeyError, TypeError):
            logger.warning('Could not determine MAC for: %s', if_name)
        else:
            logger.debug("Found link: %s", link)
            if len(link) > 1:
                logger.warning(
                    'Conflict: Multiple interfaces found for IP: %s!', ip)
            mac_addr = link[0].get('addr')
        return mac_addr





import logging

import re

def validate_id(id):
    if isinstance(id, str):
        r = re.compile('(^(?!\-)[a-z0-9\-]+(?<!\-)$)')
        return id if r.match(id) else False


logger = logging.getLogger(__name__)

data_types = [
    'integer',
    'float', 
    'boolean', 
    'string', 
    'enum', 
    'color',
]

"""
meta dict

meta = {
    'mainkeyid1' : {
        'name'  : name,
        'value' : value,
        'subkeys' = {
            'subkeyid1' : {
                'key' : name},
                'value' : value,
            },
        },
    },
    'mainkeyid2' : {
        'name'  : name,
        'value' : value,
    },
}
"""

class Property_Base(object):

    def __init__(self, node, id, name=None, settable=False, retained=True, qos=1, unit=None, data_type=None, data_format=None, value=None, set_value=None, tags=[], meta={}):
        if validate_id (id) is False:
            logger.error('Property ID not valid {}'.format(id))
            assert validate_id(id),'Property ID is not valid {}'.format(id)

        self.id = id
        self.name = name
        self.retained = retained
        self.qos = qos
        self.unit = unit
        self.node = node
        self.topic = node.topic
        self.tags = tags
        self.meta = meta 
        
        assert data_type in data_types
        self.data_type = data_type
        self.data_format = data_format
        
        self.settable = settable
        if settable: # must provide a function to call to set the value
            assert(set_value)
            self.set_value = set_value

        self._value = value

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value, retain=True, qos=1):
        if self.validate_value(value):
            self._value = value
            self.publish (self.topic,self.get_payload_from_value(value),retain,qos)
            logger.debug ('Value set to:   {}'.format(value))
        else:
            logger.warn ('Invalid Value: Device {} Node {} Property {} Value {}'.format(self.node.device.name,self.node.name,self.name,value))

    def validate_value(self,value):
        return True # override as needed

    def get_value_from_payload(self,payload): #convert payload to a value we can use
        return payload
    
    def get_payload_from_value(self,value): #convert value to a payload for homie
        return value
    
    @property
    def topic(self):
        return self._topic

    @topic.setter
    def topic(self, parent_topic):
        self._topic =  "/".join([parent_topic,self.id])

    def publish(self, topic, payload, retain, qos):
        self.node.property_publisher (topic,payload,retain,qos)

    def publish_attributes(self, retain=True, qos=1):
        self.publish ("/".join((self.topic, "$name")), self.name, retain, qos)
        self.publish ("/".join((self.topic, "$settable")), str(self.settable).lower(), retain, qos)
        self.publish ("/".join((self.topic, "$retained")), str(self.retained).lower(), retain, qos)
        if self.unit:
            self.publish ("/".join((self.topic, "$unit")), self.unit, retain, qos)
        if self.data_type:
            self.publish ("/".join((self.topic, "$datatype")), self.data_type, retain, qos)
        if self.data_format:
            self.publish ("/".join((self.topic, "$format")), self.data_format, retain, qos)

        if self.value is not None: # publish value if known, by setting it
            self.value = self.value

        self.publish_tags (retain, qos)
        self.publish_meta (retain, qos)

    def publish_tags(self, retain=True, qos=1):
        tags = ','.join(self.tags)
        if tags is not '':
            self.publish ("/".join((self.topic, "$tags")), tags, retain, qos)
    
    def publish_meta(self, retain=True, qos=1):
        main_keys = []
        for key in self.meta:
            main_keys.append(key)
        
        if len(main_keys) > 0:
            self.publish ("/".join((self.topic, "$meta", "$mainkey-ids")), ','.join(main_keys), retain, qos)

        for main_key in main_keys:
            settings = self.meta [main_key]
            self.publish ("/".join((self.topic, "$meta", main_key, "$key")), settings ['name'], retain, qos)
            self.publish ("/".join((self.topic, "$meta", main_key, "$value")), settings ['value'], retain, qos)

            if 'subkeys' in settings:
                sub_keys = []
                for key in settings ['subkeys']:
                    sub_keys.append(key)
        
                self.publish ("/".join((self.topic, "$meta", main_key, "$subkey-ids")), ','.join(sub_keys), retain, qos)

                for sub_key in sub_keys:
                    self.publish ("/".join((self.topic, "$meta", main_key, sub_key, "$key")), settings ['subkeys'] [sub_key] ['name'], retain, qos)
                    self.publish ("/".join((self.topic, "$meta", main_key, sub_key, "$value")), settings ['subkeys'] [sub_key] ['value'], retain, qos)

    def get_subscriptions(self): # subscribe to the set topic
        if self.settable:
            return {"/".join((self.topic, "set")) : self.set_message_handler}
        else:   
            return {}

    def set_message_handler(self,topic,payload):
        logger.info ('MQTT Property Message:  Topic {}, Payload {}'.format(topic,payload))
        self.process_set_message(topic,payload)

    def process_set_message(self,topic,payload): #override as needed
        value = self.get_value_from_payload(payload)

        if value is not None:
            if self.validate_value(value):
                self.value = value
                self.set_value(value) # call function to actually change the value
            else:
                logger.warning ('Payload value not valid for property for topic {}, payload is {}'.format(topic,payload))
        else:
            logger.warning ('Unable to convert payload for property topic {}, payload is {}'.format(topic,payload))





#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_contact import Node_Contact
from homie.node.property.property_contact import Property_Contact
import logging

logger = logging.getLogger(__name__)


class Device_Contact(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None):

        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_Contact(self,id='contact'))

        self.start()

    def update_contact(self,state):
        self.get_node('contact').update_contact(state)
        logger.debug ('Contact Update {}'.format(state))





from .property_float import Property_Float

tags = ['Lighting','Dimmable']

class Property_Dimmer(Property_Float):

    def __init__(self,node, id='dimmer', name = 'Dimmer', settable = True, retained = True, qos=1, unit = '%', data_type= None, data_format = '0:100', value = None, set_value=None, tags=tags, meta={}):
        
        super().__init__(node,id,name,settable,retained,qos,unit,data_type,data_format,value,set_value,tags,meta)

    def get_value_from_payload(self,payload):
        try:
            return round(float(payload)*100,0)
        except:
            return None

    def get_payload_from_value(self,value): 
        try:
            return round(value/100,2)
        except:
            return None

'''
from .property_integer import Property_Integer

tags = ['Lighting','Dimmable']

class Property_Dimmer(Property_Integer):

    def __init__(self,node, id='dimmer', name = 'Dimmer', settable = True, retained = True, qos=1, unit = '%', data_type= None, data_format = '0:100', value = None, set_value=None, tags=tags, meta={}):
        
        super().__init__(node,id,name,settable,retained,qos,unit,data_type,data_format,value,set_value,tags,meta)
'''


from .property_base import Property_Base
import logging

logger = logging.getLogger(__name__)

class Property_Integer(Property_Base):

    def __init__(self, node, id, name, settable=True, retained=True, qos=1, unit=None, data_type='integer', data_format=None, value=None, set_value=None, tags=[], meta={}):
        super().__init__(node,id,name,settable,retained,qos,unit,'integer',data_format,value,set_value,tags,meta)

        if data_format:
            range = data_format.split(':')
            self.low_value = int(range[0])
            self.high_value = int(range[1])
        else:
            self.low_value = None
            self.high_value = None

    def validate_value(self, value):
        valid = True

        if self.low_value is not None and value < self.low_value:
            valid = False
        if self.high_value is not None and value > self.high_value:
            valid = False

        return valid

    def get_value_from_payload(self,payload):
        try:
            return int(payload)
        except:
            return None

 


#!/usr/bin/env python


'''

Base MQTT Client for a Homie device

To allow for easy use of different MQTT clients

'''
from homie.support.network_information import Network_Information
network_info = Network_Information()

import logging
logger = logging.getLogger(__name__)
logger.setLevel('INFO')

MQTT_SETTINGS = {
    'MQTT_BROKER' : None,
    'MQTT_PORT' : 1883,
    'MQTT_USERNAME' : None,
    'MQTT_PASSWORD' : None,
    'MQTT_KEEPALIVE' : 60,
    'MQTT_CLIENT_ID' : None,
    'MQTT_SHARE_CLIENT' : False,
}

class MQTT_Base (object):

    def __init__(self, mqtt_settings):
        logger.debug('MQTT client Settings {}'.format(mqtt_settings))

        self.using_shared_mqtt_client = mqtt_settings ['MQTT_SHARE_CLIENT']

        self.mqtt_settings = mqtt_settings

        self._mqtt_connected = False

        self.ip_address = None
        self.mac_address = None

        self.homie_devices = []

    @property
    def mqtt_connected(self):
        return self._mqtt_connected

    @mqtt_connected.setter
    def mqtt_connected(self,connected):
        if connected != self._mqtt_connected:
            logger.debug("MQTT Connected {} ".format(connected))
            self._mqtt_connected = connected
            for device in self.homie_devices:
                if device.start_time is not None:
                    device.mqtt_on_connection(connected)

    def connect(self): #called by the device when its ready for the mqtt client to start, subclass to provide
        logger.debug("MQTT Connecting to {} as client {}".format(self.mqtt_settings ['MQTT_BROKER'],self.mqtt_settings['MQTT_CLIENT_ID']))

    def publish(self, topic, payload, retain, qos): #subclass to provide
        logger.debug('MQTT publish topic: {}, payload: {}, retain {}, qos {}'.format(topic,payload,retain,qos))

    def subscribe(self, topic, qos): #subclass to provide
        logger.debug('MQTT subscribe  topic: {}, qos {}'.format(topic,qos))

    def unsubscribe(self, topic): #subclass to provide
        logger.debug('MQTT unsubscribe  topic: {}'.format(topic))

    def set_will(self,will,topic,retain,qos): #subclass to provide
        logger.debug ('MQTT set will {}, topic {}'.format(will,topic))

    def get_mac_ip_address(self):
        if self.ip_address is None:
            self.ip_address = network_info.get_local_ip (self.mqtt_settings ['MQTT_BROKER'],self.mqtt_settings ['MQTT_PORT'])

        if self.mac_address is None:
            self.mac_address = network_info.get_local_mac_for_ip(self.ip_address)

        return self.mac_address, self.ip_address

    def _on_message(self,topic,payload,retain,qos):
        logger.debug ('MQTT On Message: Topic {}, Payload {} Reatin {} QOS {}'.format(topic,payload,retain,qos))
        for device in self.homie_devices:
            if device.start_time is not None: #device is ready
                try:
                    device.mqtt_on_message(topic,payload,retain==1,qos) 
                except:
                    logger.exception('on_message error')

    def _on_disconnect(self,rc):
        logger.debug ('MQTT On Disconnect: Result Code {}'.format(rc))
            
    def add_device(self,device):
        self.homie_devices.append(device)

    def remove_device(self,device): # not tested
        del self.homie_devices [device]





#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_switch import Node_Switch
import logging

logger = logging.getLogger(__name__)

class Device_Switch(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None):
        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_Switch(self,id='switch',set_switch=self.set_switch))

        self.start()

    def update_switch(self,onoff): #sends updates to clients
        self.get_node('switch').update_switch(onoff)
        logger.debug ('Switch Update {}'.format(onoff))

    def set_switch(self,onoff):#received commands from clients
        logger.debug ('Switch Set {}'.format(onoff))

        




from .property_base import Property_Base

class Property_String(Property_Base):

    def __init__(self, node, id, name, settable=False, retained=True, qos=1, unit=None, data_type='string', data_format=None, value=None, set_value=None, tags=[], meta={}):
        
        super().__init__(node,id,name,settable,retained,qos,unit,'string',data_format,value,set_value, tags, meta)





from .node_base import Node_Base

from homie.node.property.property_boolean import Property_Boolean


class Node_Boolean(Node_Base):

    def __init__(self, device, id='boolean', name='Boolean', type_='boolean', retain=True, qos=1, set_boolean=None): 
        super().__init__(device,id,name,type_,retain,qos)

        assert (set_boolean) # must provide a function to set the value of the switch

        self.add_property (Property_Boolean (self,set_value = set_boolean))

    def update_boolean(self,boolean):
        self.get_property ('boolean').value = boolean


 




import logging
from .property_enum import Property_Enum

logger = logging.getLogger(__name__)

class Property_Speed(Property_Enum):

    def __init__(self, node, id='speed', name='Speed', settable=True, retained=True, qos=1, unit=None, data_type='enum', data_format='OFF,LOW,MEDIUM,HIGH', value=None, set_value=None, tags=[], meta={}):

        super().__init__(node,id,name,settable,retained,qos,unit,data_type,data_format,value,set_value, tags, meta)




from homie.device_base import Device_Base
from homie.node.node_base import Node_Base

class Device_Status(Device_Base):
    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None):

        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        node = (Node_Base(self,'status','Status','status'))
        self.add_node (node)

        self.register_status_properties(node)

        self.start()

    def register_status_properties(self, node):
        raise RuntimeError('Override in subclass')



from homie.support.helpers import validate_id

class Node_Base(object):

    def __init__(self, device, id, name, type_, retain=True, qos=1):
        assert validate_id(id),'Node ID {} is not valid'.format(id)
        assert device

        self.id = id
        self.name = name
        self.type = type_
        self.device = device

        self.retain = retain
        self.qos = qos

        self.properties = {}

        self.topic = self.device.topic

        self.published = False

    @property
    def topic(self):
        return self._topic

    @topic.setter
    def topic(self, parent_topic):
        self._topic =  "/".join([parent_topic,self.id])

    def add_property(self, property_):
        #assert self.properties [property_.id] == None
        assert property_.id not in self.properties

        self.properties [property_.id] = property_

        if self.published: #need to update publish property changes
            self.publish_properties()

    def remove_property(self, property_id):
        property_ = self.properties [property_id]
        del self.properties [property_id]

        if self.device.start_time is not None: #running, publish property changes
            self.publish_properties()
            property_.publish_attributes(False,1)

    def get_property(self, property_id):
        if property_id in self.properties:
            return self.properties [property_id]
        else:
            return None

    def set_property_value(self, property_id, value):
        self.get_property (property_id).value = value

    def publish(self,topic,payload,retain,qos):
        self.device.publish (topic,payload,retain,qos)

    def property_publisher(self,topic,payload,retain,qos): # properties use this to publish
        if self.published: # only publish if the node has been published
            self.device.publish (topic,payload,retain,qos)

    def publish_attributes(self, retain=True, qos=1):
        self.publish ("/".join((self.topic, "$name")), self.name, retain, qos)
        self.publish ("/".join((self.topic, "$type")), self.type, retain, qos)

        self.publish_properties()
        
    def publish_properties(self, retain=True, qos=1):
        properties = ",".join(self.properties.keys())
        self.publish ("/".join((self.topic, "$properties")), properties, retain, qos)

        self.published = True # node basics published

        for _,property_ in self.properties.items():
            #print ('NODE PUBLISH PROP ',property_.name)
            property_.publish_attributes(retain, qos)        
        
    def get_subscriptions(self):
        subscriptions = {}
        
        for _,property_ in self.properties.items():
            subscriptions.update (property_.get_subscriptions())

        return subscriptions




#!/usr/bin/env python

from homie.device_temperature import Device_Temperature
from homie.node.property.property_humidity import Property_Humidity

import logging

logger = logging.getLogger(__name__)

class Device_Temperature_Humidity(Device_Temperature):
    def register_status_properties(self, node):
        super(Device_Temperature_Humidity, self).register_status_properties(node)

        self.humidity = Property_Humidity(node)
        node.add_property(self.humidity)

    def update_humidity(self,humidity):
        logger.info ('Updated Humidity {}'.format(humidity))
        self.humidity.value = humidity




from .property_base import Property_Base

class Property_Boolean(Property_Base):

    def __init__(self, node, id='boolean', name='Boolean', settable=True, retained=True, qos=1, unit=None, data_type='boolean', data_format=None, value=None, set_value=None, tags=[], meta={}):
        super().__init__(node,id,name,settable,retained,qos,unit,'boolean',data_format,value,set_value,tags,meta)

    def validate_value(self, value):
        return isinstance(value,bool)

    def get_value_from_payload(self,payload):
        if payload == 'true':
            return True 
        elif payload == 'false':
            return False
        else:
            return None

    def get_payload_from_value(self,value): #convert value to a payload for homie
        return value and "true" or "false"
        





#!/usr/bin/env python

import sys
import time
import atexit
from datetime import datetime

import homie

from homie.support.helpers import validate_id
from homie.mqtt.homie_mqtt_client import connect_mqtt_client
from homie.support.repeating_timer import Repeating_Timer

import logging
logger = logging.getLogger(__name__)
logger.setLevel('INFO')

instance_count = 0 # used to track the number of device instances to allow for changing the default device id

repeating_timer = None # use common timer between all devices for updating state

DEVICE_STATES = [
    "init",
    "ready",
    "disconnected",
    "sleeping",
    "alert",
    "lost",
]

HOMIE_SETTINGS = {
    'version' : '4.0.0',
    'topic' : 'homie',
    'fw_name' : 'homie4',
    'fw_version' : homie.__version__,
    'update_interval' : 60,
    'implementation' : sys.platform,
}

EXTENSIONS = [ # extension we suport
    'stats',
    'firmware',
    'meta'
]

EXTENSION_IDENTIFIERS = {
    'stats' : 'org.homie.legacy-stats:0.1.1:[4.x]',
    'firmware' : 'org.homie.legacy-firmware:0.1.1:[4.x]',
    'meta' : 'eu.epnw.meta:1.1.0:[3.0.1;4.x]',
}

class Device_Base(object):

    def __init__(self, device_id=None, name=None, homie_settings={}, mqtt_settings={}, extensions=['stats','firmware','meta']):
        global instance_count
        instance_count = instance_count + 1
        self.instance_number = instance_count

        self._mqtt_connected = False # mqtt connection status, true = connected

        if device_id is None:
            device_id=self.generate_device_id()

        assert validate_id(device_id), 'Invalid device id {}'.format(device_id)
        self.device_id = device_id

        assert name
        self.name = name

        for extension in extensions:
            assert (extension in EXTENSIONS)
        self.extensions = extensions

        self._state = "init"

        self.homie_settings = self._homie_validate_settings (homie_settings)
        self.topic = "/".join((self.homie_settings ['topic'], self.device_id))

        #may need a way to set these
        self.retained = True
        self.qos = 1

        self.nodes = {}

        self.start_time = None

        self.nodes_published = False

        self.mqtt_client = connect_mqtt_client(self,mqtt_settings)

        self.mqtt_subscription_handlers = {}

        atexit.register(self.cleanup)

    def cleanup(self):
        logger.debug ('clean up')
        self.state = 'disconnected'

    def generate_device_id(self):
        #logger.debug ('Device instances {}'.format(instance_count))
        #return "{:02x}".format(get_mac())+"{:04d}".format(instance_count)
        return "device{:04d}".format(self.instance_number)

    def start(self): # called after the device has been built with nodes and properties
        logger.debug ('Device startup')
        self.start_time = time.time()

        if 'stats' in self.extensions:
            global repeating_timer
            if repeating_timer == None:
                repeating_timer = Repeating_Timer(self.homie_settings['update_interval'])

            repeating_timer.add_callback (self.publish_uptime)

        if self.mqtt_client.mqtt_connected: #run start up tasks if mqtt is ready, else wait for on_connect message from mqtt client
            self.mqtt_on_connection(True)

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, state, retain = True, qos = 1):
        if state in DEVICE_STATES:
            self._state = state
            self.publish( "/".join((self.topic, "$state")),self._state, retain, qos)
        else:
            logger.warning ('Homie Invalid device state {}'.format(state))

    def publish_attributes(self, retain=True, qos=1):
        self.publish("/".join((self.topic, "$homie")),self.homie_settings ['version'], retain, qos)
        self.publish("/".join((self.topic, "$name")),self.name, retain, qos)
        self.publish("/".join((self.topic, "$implementation")),self.homie_settings ['implementation'], retain, qos)

        self.publish_extensions(retain, qos)

        self.state = 'ready'

    def publish_extensions(self, retain = True, qos = 1):
        extensions = ','.join(self.extensions)

        if extensions is not '':
            self.publish("/".join((self.topic, "$extensions")),extensions, retain, qos)

        if 'stats' in self.extensions:
            self.publish_statastics(retain, qos)

        if 'firmware' in self.extensions:
            self.publish_firmware(retain, qos)


    def publish_firmware(self, retain = True, qos = 1):
        mac,ip = self.mqtt_client.get_mac_ip_address()

        self.publish("/".join((self.topic, "$localip")),ip, retain, qos)
        self.publish("/".join((self.topic, "$mac")),mac, retain, qos)
        self.publish("/".join((self.topic, "$fw/name")),self.homie_settings ['fw_name'], retain, qos)
        self.publish("/".join((self.topic, "$fw/version")),self.homie_settings ['fw_version'], retain, qos)
        self.publish("/".join((self.topic, "$implementation")),self.homie_settings ['implementation'], retain, qos)

    def publish_statastics(self, retain = True, qos = 1):
        self.publish("/".join((self.topic, "$stats/interval")),self.homie_settings ['update_interval'], retain, qos)
        self.publish("/".join((self.topic, "$stats/uptime")),time.time()-self.start_time, retain, qos)
        self.publish("/".join((self.topic, "$stats/lastupdate")),datetime.now().strftime("%d/%m/%Y %H:%M:%S"), retain, qos)

    def publish_uptime(self, retain=True, qos=1):
        self.publish("/".join((self.topic, "$stats/uptime")),time.time()-self.start_time, retain, qos)
        self.publish("/".join((self.topic, "$stats/lastupdate")),datetime.now().strftime("%d/%m/%Y %H:%M:%S"), retain, qos)

    def add_subscription(self,topic,handler,qos=0): #subscription list to the required MQTT topics, used by properties to catch set topics
        self.mqtt_subscription_handlers [topic] = handler
        self.mqtt_client.subscribe (topic,qos)
        logger.debug ('MQTT subscribed to {}'.format(topic))

    def remove_subscription(self,topic):
        self.mqtt_client.unsubscribe (topic)
        del self.mqtt_subscription_handlers [topic]
        logger.debug ('MQTT unsubscribed to {}'.format(topic))

    def subscribe_topics(self):
        logger.debug('Device subscribing to topics')
        self.add_subscription ("/".join((self.topic, "$broadcast/#")),self.broadcast_handler) #get the broadcast events

        for _,node in self.nodes.items():
            for topic,handler in node.get_subscriptions().items():
                self.add_subscription(topic,handler)

    def add_node(self,node):
        self.nodes [node.id] = node

        if self.nodes_published: #update, publish property changes
            self.publish_nodes(self.retained, self.qos)

    def remove_node(self, node_id): # not tested, needs work removing topics
        del self.nodes [node_id]

        if self.nodes_published: #update, publish property changes
            self.publish_nodes(retain = False)

    def get_node(self,node_id):
        if node_id in self.nodes:
            return self.nodes [node_id]
        else:
            return None

    def publish_nodes(self, retain=True, qos=1):
        nodes = ",".join(self.nodes.keys())
        self.publish("/".join((self.topic, "$nodes")), nodes, retain, qos)

        self.nodes_published = True

        for _,node in self.nodes.items():
            node.publish_attributes(retain, qos)

    def broadcast_handler(self,topic,payload):#TBD
        logger.debug ('Device MQTT Homie Broadcast:  Topic {}, Payload {}'.format(topic,payload))

    def publish(self, topic, payload, retain, qos):
        logger.debug('Device MQTT publish topic: {}, retain {}, qos {}, payload: {}'.format(topic,retain,qos,payload))
        self.mqtt_client.publish(topic, payload, retain=retain, qos=qos)

    def _homie_validate_settings(self,settings):
        if settings is not None:
            for setting,value in HOMIE_SETTINGS.items():
                logger.debug('Homie settings {} {}'.format(setting,value))
                if not setting in settings:
                    settings [setting] = HOMIE_SETTINGS [setting]
        else:
            settings = HOMIE_SETTINGS

        return settings

    def mqtt_on_connection(self,connected):
        logger.debug("Device MQTT Connected state is {}".format(connected))
        #print("Device MQTT Connected state is {}".format(connected))

        if connected:
            if self._mqtt_connected is False:
                self._mqtt_connected = True
                self.publish_attributes()
                self.publish_nodes()
                self.subscribe_topics()
                if self.mqtt_client.using_shared_mqtt_client is False or self.instance_number == 1: # only set last will if NOT using shared client or if using shared client and this is the first device instance
                    self.mqtt_client.set_will("/".join((self.topic, "$state")), "lost", retain=True, qos=1)
                    logger.debug ('Device setting last will')
        else:
            self._mqtt_connected = False

    def mqtt_on_message(self, topic, payload, retain, qos):
        if topic in self.mqtt_subscription_handlers:
            logger.debug ('Device MQTT Message: Topic {}, Payload {} Retain {}  QOS {}'.format(topic,payload,retain,qos)) #for logging only, topic and handler for subsriptions above
            if retain is False:
                self.mqtt_subscription_handlers [topic] (topic, payload)
            else:
                logger.warn ('Device MQTT Message received with RETAIN TRUE: Topic {}, Payload {} Retain {}  QOS {}'.format(topic,payload,retain,qos)) #for logging only, topic and handler for subsriptions above



import re



''''

Topic IDs

An MQTT topic consists of one or more topic levels, separated by the slash character (/). A topic level ID MAY contain lowercase letters from a to z, numbers from 0 to 9 as well as the hyphen character (-).
A topic level ID MUST NOT start or end with a hyphen (-). The special character $ is used and reserved for Homie attributes. The underscore (_) is used and reserved for Homie node arrays.

'''

def validate_id(id):
    if isinstance(id, str):
        r = re.compile('(^(?!\-)[a-z0-9\-]+(?<!\-)$)')
        return id if r.match(id) else False


if __name__ == '__main__':

    print(validate_id('Xx'))    
    print(validate_id(3368967))


from .property_boolean import Property_Boolean


tags = ['Contact']


class Property_Contact(Property_Boolean):

    def __init__(self, node, id='contact', name = 'Contact', settable = False, retained = True, qos=1, unit = None, data_type= None, data_format = None, value = None, set_value=None, tags=tags , meta={}):
        super().__init__(node, id,name,settable,retained,qos,unit,data_type,data_format,value,set_value, tags, meta)
    
    def validate_value(self, value):
        return value in ['OPEN','CLOSED']
    
    def get_payload_from_value(self,value): 
        if value == 'OPEN':
            return 'true'           
        else:
            return 'false'


'''
from .property_enum import Property_Enum

class Property_Contact(Property_Enum):

    def __init__(self, node, id='contact', name='Contact', settable=False, retained=False, qos=1, unit=None, data_type='enum', data_format='OPEN,CLOSED', value=None, set_value=None, tags=[], meta={}):
        
        super().__init__(node, id,name,settable,retained,qos,unit,data_type,data_format,value,set_value)

 '''



from .node_base import Node_Base

from homie.node.property.property_integer import Property_Integer


class Node_Integer(Node_Base):

    def __init__(self, device, id='integer', name='State', type_='integer', retain=True, qos=1, set_value=None): 
        super().__init__(device,id,name,type_,retain,qos)

        assert (set_value)

        self.add_property (Property_Integer (self,'integer','Integer',set_value=set_value))

    def update_value(self,value):
        self.get_property('integer').value = value



from .property_float import Property_Float

class Property_Temperature(Property_Float):

    def __init__(self, node, id='temperature', name = 'Temperature', settable = False, retained = True, qos=1, unit = None, data_type= None, data_format = None, value = None, set_value=None, tags=[], meta={}):
        
        super().__init__(node,id,name,settable,retained,qos,unit,data_type,data_format,value,set_value, tags, meta)

 




from .property_integer import Property_Integer

class Property_Humidity(Property_Integer):

    def __init__(self,node, id='humidity', name = 'Humidity', settable = False, retained = True, qos=1, unit = '%', data_type= None, data_format = '0:100', value = None, set_value=None, tags=[], meta={}):
        
        super().__init__(node,id,name,settable,retained,qos,unit,data_type,data_format,value,set_value, tags, meta)

 




#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_dimmer import Node_Dimmer
import logging

logger = logging.getLogger(__name__)
logger.setLevel('INFO')


class Device_Dimmer(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None):
        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_Dimmer(self,id='dimmer',set_dimmer=self.set_dimmer))

        self.start()

    def update_dimmer(self,percent): #sends updates to clients
        self.get_node('dimmer').update_dimmer(round(percent,0))
        logger.debug ('Dimmer Update {}'.format(percent))

    def set_dimmer(self,percent):#received commands from clients
        logger.debug ('Dimmer Set {}'.format(round(percent,0)))



import logging
from .property_base import Property_Base

logger = logging.getLogger(__name__)

class Property_Enum(Property_Base):

    def __init__(self, node, id, name, settable=True, retained=True, qos=1, unit=None, data_type='enum', data_format=None, value=None, set_value=None, tags=[], meta={}):
        assert(data_format)
        super().__init__(node,id,name,settable,retained,qos,unit,data_type,data_format,value,set_value, tags, meta)

        self.enum_list = data_format.split(',')

    def validate_value(self, value):
        return value in self.enum_list




from .property_float import Property_Float

class Property_Setpoint(Property_Float):

    def __init__(self, node, id=None, name=None, settable=True, retained=True, qos=1, unit=None, data_type=None, data_format=None, value=None, set_value=None, tags=[], meta={}):
        
        super().__init__(node,id,name,settable,retained,qos,unit,data_type,data_format,value,set_value, tags, meta)

 



import logging
from .property_base import Property_Base

logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.DEBUG)

# ********** not completed

class Property_Color(Property_Base):

    def __init__(self, node, id, name, settable=True, retained=True, qos=1, unit=None, data_type='color', data_format=None, value=None, set_value=None, tags=[], meta={}):

        super().__init__(node,id,name,settable,retained,qos,unit,'color',data_format,value,set_value,tags,meta)

        # check valid data format provided

    def message_handler(self,topic,payload):
        pass




from .node_base import Node_Base

from homie.node.property.property_dimmer import Property_Dimmer


class Node_Dimmer(Node_Base):

    def __init__(self, device, id='dimmer', name='Dimmer', type_='dimmer', retain=True, qos=1, set_dimmer=None): 
        super().__init__(device,id,name,type_,retain,qos)

        assert (set_dimmer) # must provide a function to set the value of the dimmer

        self.add_property (Property_Dimmer (self,set_value=set_dimmer))

    def update_dimmer(self,percent):
        self.get_property('dimmer').value = percent



from .node_base import Node_Base

from homie.node.property.property_enum import Property_Enum


class Node_State(Node_Base):

    def __init__(self, device, id='state', name='State', type_='state', retain=True, qos=1, state_values=None,set_state=None): 
        assert (state_values)
        assert (set_state)

        super().__init__(device,id,name,type_,retain,qos)

        self.add_property (Property_Enum (self,'state','State',data_format=state_values,set_value=set_state))

    def update_state(self,state):
        self.get_property('state').value = state



from .node_base import Node_Base

from homie.node.property.property_contact import Property_Contact


class Node_Contact(Node_Base):

    def __init__(self, device, id='contact', name='Contact', type_='contact', retain=True, qos=1): 
        super().__init__(device,id,name,type_,retain,qos)

        self.add_property (Property_Contact (self,'contact'))

    def update_contact(self,contact):
        self.get_property('contact').value = contact



from .property_boolean import Property_Boolean


tags = ['Switch']


class Property_Switch(Property_Boolean):

    def __init__(self, node, id='switch', name = 'Switch', settable = True, retained = True, qos=1, unit = None, data_type= None, data_format = None, value = None, set_value=None, tags=tags , meta={}):
        super().__init__(node, id,name,settable,retained,qos,unit,data_type,data_format,value,set_value, tags, meta)
    
    def validate_value(self, value):
        return value in ['ON','OFF']
    
    def get_value_from_payload(self,payload):
        if payload == 'true':
            return 'ON'
        elif payload == 'false':
            return 'OFF'
        else:
            return None

    def get_payload_from_value(self,value): 
        if value == 'ON':
            return 'true'           
        else:
            return 'false'


#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_state import Node_State

import logging

logger = logging.getLogger(__name__)


class Device_State(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None,state_values=None):

        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_State(self,id='state',name='State',state_values=state_values,set_state=self.set_state))

        self.start()

    def update_state(self,state):
        self.get_node('state').update_state(state)
        logger.debug ('State Update {}'.format(state))


    def set_state(self,state):#received commands from clients
        # subclass must override and provide logic to set the device
        logger.debug ('State Set {}'.format(state))




from .property_base import Property_Base

import datetime

class Property_DateTime(Property_Base):

    def __init__(self, node, id, name, settable=False, retained=True, qos=1, unit=None, data_type='datetime', data_format=None, value=None, set_value=None, tags=[], meta={}):
        
        super().__init__(node,id,name,settable,retained,qos,unit,'datetime',data_format,value,set_value,tags,meta)

    def validate_value(self, value):
        try:
            date_time_obj = datetime.datetime.strptime(date_time_str, '%Y-%m-%d %H:%M:%S.%f')
            return True
        except:
            return False

    def get_value_from_payload(self,payload):
        try:
            return datetime.datetime.strptime(payload, '%Y-%m-%d %H:%M:%S.%f')
        except:
            return None
    





#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_integer import Node_Integer

import logging

logger = logging.getLogger(__name__)


class Device_Integer(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None):

        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        self.add_node(Node_Integer(self,id='integer',name='Integer',set_value=self.set_value))

        self.start()

    def update_value(self,value):
        self.get_node('integer').update_value(value)
        logger.debug ('Integer Update {}'.format(value))


    def set_value(self,value):#received commands from clients
        # subclass must override and provide logic to set the device
        logger.debug ('Integer Set {}'.format(value))




import paho.mqtt.client as mqtt

# The callback for when the client receives a CONNACK response from the server.
def on_connect(client, userdata, flags, rc):
    print("Connected with result code "+str(rc))

    # Subscribing in on_connect() means that if we lose the connection and
    # reconnect then subscriptions will be renewed.
    client.subscribe("$SYS/#")

# The callback for when a PUBLISH message is received from the server.
def on_message(client, userdata, msg):
    print(msg.topic+" "+str(msg.payload))

def on_disconnect(client, userdata, flags, rc):
    print("Connected with result code "+str(rc))

client = mqtt.Client()
client.on_connect = on_connect
client.on_message = on_message
client.on_disconnect = on_disconnect

client.connect("openhab", 1883, 60)

# Blocking call that processes network traffic, dispatches callbacks and
# handles reconnecting.
# Other loop*() functions are available that give a threaded interface and a
# manual interface.
client.loop_forever()


import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name='Homie4',
    version='0.1.8',
    description='Homie 4.0.0 Implementation',
    author='Michael Cumming',
    author_email='mike@4831.com',
    long_description=long_description,
    long_description_content_type="text/markdown",      
    url='https://github.com/mjcumming/Homie',
    keywords = ['HOMIE','MQTT'],  
    packages=setuptools.find_packages(exclude=("test",)),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],      
    install_requires=[
        'paho-mqtt>=1.3.0',
        'netifaces>=0.10.6',
    ]      
)



from .node_base import Node_Base

from homie.node.property.property_switch import Property_Switch


class Node_Switch(Node_Base):

    def __init__(self, device, id='switch', name='Switch', type_='switch', retain=True, qos=1, set_switch=None): 
        super().__init__(device,id,name,type_,retain,qos)

        assert (set_switch) # must provide a function to set the value of the switch

        self.add_property (Property_Switch (self,set_value = set_switch))

    def update_switch(self,onoff):
        self.get_property ('switch').value = onoff


 




#!/usr/bin/env python

from homie.device_base import Device_Base
from homie.node.node_base import Node_Base

from homie.node.property.property_setpoint import Property_Setpoint
from homie.node.property.property_temperature import Property_Temperature
from homie.node.property.property_humidity import Property_Humidity
from homie.node.property.property_enum import Property_Enum
from homie.node.property.property_string import Property_String

import logging
logger = logging.getLogger(__name__)
logger.setLevel('INFO')

# example
THERMOSTAT_SETTINGS = {
    'current_temperature' : 70,
    'current_humidity' : 30,
    'cool_setpoint' : 70,
    'max_cool_setpoint' : 90,
    'min_cool_setpoint' : 60,
    'heat_setpoint' : 70,
    'max_heat_setpoint' : 90,
    'min_heat_setpoint' : 60,    
    'fan_mode' : 'On',
    'fan_modes' : ['On','Auto'],
    'hold_mode' : 'Schedule',
    'hold_modes' : ['Schedule','Temporary','Permanent'],
    'system_mode' : 'Off',
    'system_modes' : ['Off','Heat','Cool','Auto'],
    'system_status' : 'Off',
    'units' : 'F',
}


class Device_Thermostat(Device_Base):

    def __init__(self, device_id=None, name=None, homie_settings=None, mqtt_settings=None, thermostat_settings=THERMOSTAT_SETTINGS):

        super().__init__ (device_id, name, homie_settings, mqtt_settings)

        node = (Node_Base(self,'controls','Controls','controls'))
        self.add_node (node)

        if 'heat_setpoint' in thermostat_settings:
            heat_setpt_limits = '{}:{}'.format(thermostat_settings['min_heat_setpoint'],thermostat_settings['max_heat_setpoint'])
            self.heat_setpoint = Property_Setpoint (node,id='heatsetpoint',name='Heat Setpoint',data_format=heat_setpt_limits,unit=thermostat_settings['units'],value=thermostat_settings['heat_setpoint'],set_value = self.set_heat_setpoint)
            node.add_property (self.heat_setpoint)

        if 'cool_setpoint' in thermostat_settings:
            cool_setpt_limits = '{}:{}'.format(thermostat_settings['min_cool_setpoint'],thermostat_settings['max_cool_setpoint'])
            self.cool_setpoint = Property_Setpoint (node,id='coolsetpoint',name='Cool Setpoint',data_format=cool_setpt_limits,unit=thermostat_settings['units'],value=thermostat_settings['cool_setpoint'],set_value = self.set_cool_setpoint)
            node.add_property (self.cool_setpoint)

        if 'system_mode' in thermostat_settings:
            self.system_mode = Property_Enum (node,id='systemmode',name='System Mode',data_format=','.join(thermostat_settings['system_modes']),value=thermostat_settings['system_mode'],set_value =self.set_system_mode)
            node.add_property (self.system_mode)

        if 'fan_mode' in thermostat_settings:
            self.fan_mode = Property_Enum (node,id='fanmode',name='Fan Mode',data_format=','.join(thermostat_settings['fan_modes']),value=thermostat_settings['fan_mode'],set_value=self.set_fan_mode)
            node.add_property (self.fan_mode)

        if 'hold_mode' in thermostat_settings:
            self.hold_mode = Property_Enum (node,id='holdmode',name='Hold Mode',data_format=','.join(thermostat_settings['hold_modes']),value=thermostat_settings['hold_mode'],set_value = self.set_hold_mode)
            node.add_property (self.hold_mode)

        node = (Node_Base(self,'status','Status','status'))
        self.add_node (node)

        self.current_temperture = Property_Temperature (node,unit=thermostat_settings['units'],value=thermostat_settings['current_temperature'])
        node.add_property (self.current_temperture)

        self.current_humidity = Property_Humidity (node,value=thermostat_settings['current_humidity'])
        node.add_property (self.current_humidity)

        self.system_status = Property_String (node,id='systemstatus',name='System Status',value=thermostat_settings['system_status'])
        node.add_property (self.system_status)

        self.start()
    
    def update_heat_setpoint(self,value):
        self.heat_setpoint.value = value

    def update_cool_setpoint(self,value):
        self.cool_setpoint.value = value
        
    def update_system_mode(self,value):
        self.system_mode.value=value
        
    def update_fan_mode(self,value):
        self.fan_mode.value=value
        
    def update_hold_mode(self,value):
        self.hold_mode.value=value
        
    def update_current_temperature(self,value):
        self.current_temperture.value=value

    def update_current_humidity(self,value):
        self.current_humidity.value=value

    def set_heat_setpoint(self,value):
        logger.debug ('Heat Set {}'.format(value))

    def set_cool_setpoint(self,value):
        logger.debug ('Cool Set {}'.format(value))
        
    def set_system_mode(self,value):
        logger.debug ('Mode Set {}'.format(value))
        
    def set_fan_mode(self,value):
        logger.debug ('Fan Set {}'.format(value))
        
    def set_hold_mode(self,value):
        logger.debug ('Hold Set {}'.format(value))



#!/usr/bin/env python

from homie.mqtt.paho_mqtt_client import PAHO_MQTT_Client

import logging
logger = logging.getLogger(__name__)

MQTT_SETTINGS = {
    'MQTT_BROKER' : None,
    'MQTT_PORT' : 1883,
    'MQTT_USERNAME' : None,
    'MQTT_PASSWORD' : None,
    'MQTT_KEEPALIVE' : 60,
    'MQTT_CLIENT_ID' : None,
    'MQTT_SHARE_CLIENT' : None,
}

mqtt_client_count = 0
    
def _mqtt_validate_settings(settings):
    settings = settings.copy()
    
    for setting,value in MQTT_SETTINGS.items():
        if not setting in settings:
            settings [setting] = MQTT_SETTINGS [setting]
        logger.debug ('MQTT Settings {} {}'.format(setting,settings [setting]))

    assert settings ['MQTT_BROKER']
    assert settings ['MQTT_PORT']

    ''' cannot use if two homie clients running from same pc
    if settings ['MQTT_CLIENT_ID'] is None or settings ['MQTT_SHARE_CLIENT'] is False:
        settings ['MQTT_CLIENT_ID'] = 'homiev3{:04d}'.format(mqtt_client_count) 
    '''
    
    return settings

common_mqtt_client = None

def connect_mqtt_client (device,mqtt_settings):
    global mqtt_client_count
    
    mqtt_settings = _mqtt_validate_settings (mqtt_settings)

    mqtt_client = None

    if mqtt_settings ['MQTT_SHARE_CLIENT'] is not True:

        logger.debug ('Using new MQTT client, number of instances {}'.format(mqtt_client_count))
        
        mqtt_client = PAHO_MQTT_Client (mqtt_settings)
        mqtt_client.connect()
        mqtt_client_count = mqtt_client_count + 1   

    else:
        logger.debug ('Using common MQTT client')

        global common_mqtt_client
        if common_mqtt_client is None:
            common_mqtt_client = PAHO_MQTT_Client (mqtt_settings)
            common_mqtt_client.connect()
            mqtt_client_count = mqtt_client_count + 1   

        mqtt_client = common_mqtt_client


    mqtt_client.add_device(device)

    return mqtt_client
   